import numpy as np

from particle import Particle


def randomize(mean):
    n = int(mean * 2 - 1)
    probs = [i/n for i in range(n+1)]
    p = np.random.uniform()
    t = [n for n,i in enumerate(probs) if i>p][0]
    return t


class ParticleGenerator:
    def __init__(self, mode_id, mean_t, left_p, x_start, x_end, y, m, r, T, bad_x_start, bad_x_end):
        self.mode_id = mode_id
        self.mean_t = mean_t
        self.p = left_p
        self.x_start = x_start
        self.x_end = x_end
        self.y = y
        self.m = m
        self.r = r
        self.T = T
        self.bad_x_start = bad_x_start
        self.bad_x_end = bad_x_end
        self.random_t = 0
        self.queue = 0
        self.prev_particle = None
        self.symmetric = True


    def get_particle(self, t):
        if self.mode_id == 0:
            if t % self.mean_t == 0:
                return self._create_particle(True),

        if self.mode_id == 1:
            if self.random_t == 0:
                self.random_t = randomize(self.mean_t)
                return self._create_particle(),
            else:
                self.random_t -= 1

        if self.mode_id == 2:
            if t % self.mean_t == 0:
                if self.queue == 0:
                    p = np.random.uniform()
                    if p <= 0.33:
                        self.queue = randomize(3)
            if self.queue > 0:
                self.queue -= 1
                return self._create_particle(True),

        if self.mode_id == 3:
            if t % self.mean_t == 0:
                new_particle = self._create_particle()
                if new_particle.x == self.x_start:
                    x = self.x_end
                else:
                    x = self.x_start
                sym_particle = Particle(x=x, y = self.y, v_x=(-1)*new_particle.v_x, r=self.r, m=self.m)
                return new_particle, sym_particle
        if self.mode_id == 4:
            if t % self.mean_t == 0:
                new_particle = self._create_particle(True)
                if new_particle.x == self.x_start:
                    x = self.x_end
                else:
                    x = self.x_start
                sym_particle = Particle(x=x, y = self.y, v_x=(-1)*new_particle.v_x, r=self.r, m=self.m)
                return new_particle, sym_particle
        return None

    def _create_particle(self, fixed_speed=False):
        p = np.random.uniform()
        if p <= self.p:
            x = self.x_start
            v_x = 1
        else:
            x = self.x_end
            v_x = -1

        if not fixed_speed:
            K = (np.random.normal(self.T, 2 * self.T / 3) - 100) / 4
            v_x *= (2 * (abs(K) + 0.001) / self.m)**0.5 / 10

        else:
            K = (self.T - 100) / 4
            v_x *= (2 * (abs(K) + 0.001) / self.m)**0.5 / 10

        if abs(v_x) < 4:
            v_x = 4 * v_x / abs(v_x)

        return Particle(x=x, y=self.y, v_x=v_x, r=self.r, m=self.m)
