import pygame
import pygame.font


class DropDownMenu:
    def __init__(self, pos, size, options, title):
        self.pos = list(pos)
        self.bg_color = (245, 245, 245)
        self.line_color = (0, 0, 0)
        self.width, self.height = size[0], size[1]
        self.options = options
        self.expanded = False
        self.cur_option = self.options[0]
        self.FONT = pygame.font.SysFont('arial', int(self.height // 2))
        self.title_surface = self.FONT.render(title, True, (0, 0, 0))

    def draw(self, screen):
        font = self.FONT
        screen.blit(self.title_surface, (self.pos[0], self.pos[1] - 3*self.height // 5))

        pygame.draw.rect(screen, self.line_color, (self.pos[0], self.pos[1], self.width, self.height), width=1, border_radius=5)

        if self.expanded:
            for i in range(len(self.options)):
                pygame.draw.rect(screen, self.bg_color,
                                 (self.pos[0], self.pos[1] + self.height * (i + 1), self.width, self.height))
                pygame.draw.line(screen, (0, 0, 0), (self.pos[0], self.pos[1] + self.height * (i + 1) + self.height),
                                 (self.pos[0] + self.width, self.pos[1] + self.height * (i + 1) + self.height), 2)
                text = font.render(self.options[i], True, (0, 0, 0))
                screen.blit(text,
                            (self.pos[0] + self.height // 5, self.pos[1] + self.height * (i + 1) + self.height // 5))

            for i in range(len(self.options)):
                pygame.draw.line(screen, self.line_color,
                                 (self.pos[0], self.pos[1] + self.height * (i + 1) + self.height),
                                 (self.pos[0] + self.width - 1, self.pos[1] + self.height * (i + 1) + self.height), 1)
        else:
            text = font.render(self.cur_option, True, (0, 0, 0))
            screen.blit(text, (self.pos[0] + self.height // 5, self.pos[1] + self.height // 5))

    def expand(self):
        self.expanded = not self.expanded

    def handle_event(self, event, margin=(0, 0)):
        if event.type == pygame.MOUSEBUTTONDOWN:
            click_coords = (event.pos[0] - margin[0], event.pos[1] - margin[1])
            if self.pos[0] <= click_coords[0] <= self.pos[0] + self.width and \
                    self.pos[1] <= click_coords[1] <= self.pos[1] + self.height:
                self.expand()
            elif self.expanded:
                for i in range(len(self.options)):
                    if self.pos[0] <= click_coords[0] <= self.pos[0] + self.width and \
                            self.pos[1] + self.height * (i + 1) <= click_coords[1] <= self.pos[1] + self.height * (
                            i + 1) + self.height:
                        self.cur_option = self.options[i]
                        self.expand()

    def get_option(self):
        return self.cur_option
