import pygame

COLOR_INACTIVE = (80, 80, 80)
COLOR_ACTIVE = (0, 0, 0)


class InputBox:
    def __init__(self, pos, size, value=0.0, title='', dtype=float):
        self.rect = pygame.Rect(pos[0], pos[1], size[0], size[1])
        self.dtype = dtype
        self.value = self.cast_to_dtype(value)
        self.default_value = value
        self.title = title
        self.color = COLOR_INACTIVE
        self.buffer = ''
        self.contents = 0
        self.text_size = int(3*size[1] / 5)
        self.FONT = pygame.font.SysFont('arial', self.text_size)
        self.txt_surface = self.FONT.render(str(value), True, self.color)
        self.title_surface = self.FONT.render(title, True, (0, 0, 0))
        self.active = False

    def handle_event(self, event, margin):
        if event.type == pygame.MOUSEBUTTONDOWN:
            # If the user clicked on the input_box rect.
            click_coords = (event.pos[0] - margin[0], event.pos[1] - margin[1])
            if self.rect.collidepoint(click_coords):
                # Toggle the active variable.
                if not self.active:
                    self.txt_surface = self.FONT.render('', True, self.color)
                self.active = not self.active
            else:
                if self.active:
                    if self.buffer != '':
                        self.value = self.cast_to_dtype(self.buffer)
                        self.buffer = ''
                    self.color = COLOR_ACTIVE if self.active else COLOR_INACTIVE
                    self.txt_surface = self.FONT.render(str(self.value), True, COLOR_INACTIVE)
                    self.active = not self.active

            self.color = COLOR_ACTIVE if self.active else COLOR_INACTIVE

        if event.type == pygame.KEYDOWN:
            if self.active:
                if event.key == pygame.K_RETURN:
                    self.value = self.cast_to_dtype(self.buffer)
                    self.buffer = ''
                    self.color = COLOR_ACTIVE if self.active else COLOR_INACTIVE
                    self.txt_surface = self.FONT.render(str(self.value), True, COLOR_INACTIVE)
                    self.active = not self.active

                elif event.key == pygame.K_BACKSPACE:
                    self.buffer = self.buffer[:-1]
                    self.txt_surface = self.FONT.render(self.buffer, True, self.color)
                else:
                    self.buffer += event.unicode
                    self.txt_surface = self.FONT.render(self.buffer, True, self.color)

    def draw(self, screen):
        # Blit the text.
        screen.blit(self.title_surface, (self.rect.x, self.rect.y - self.text_size))

        screen.blit(self.txt_surface, (self.rect.x + self.rect.width // 10, self.rect.y + self.text_size / 2))
        # Blit the rect.
        pygame.draw.rect(screen, self.color, self.rect, 1, border_radius=5)

    def cast_to_dtype(self, v):
        try:
            value = self.dtype(v)
        except ValueError:
            value = self.default_value
        return value

    def set_value(self, v):
        self.value = v
        self.txt_surface = self.FONT.render(str(self.value), True, self.color)

